@@ -7,6 +7,7 @@ import concurrent.futures
 import configparser
 import copy
 import dataclasses
+import itertools
 import logging
 import logging.handlers
 import os
@@ -69,13 +70,13 @@ def get_stream_list(
     if not (allstreams := outjson.get("streams")):
         return []
     stream_list = []
+    BAD_VALS: Final[set[str | int]] = {"N/A", "unknown"}
     for s in allstreams:
         val_list = []
-        bad_vals: set[str | int] = {"N/A", "unknown"}
         ituple = (
-            ("", s, q_tuple[0], bad_vals),
-            ("tags: ", s.get("tags", {}), q_tuple[1], bad_vals),
-            ("disposition: ", s.get("disposition", {}), q_tuple[2], bad_vals | {0}),
+            ("", s, q_tuple[0], BAD_VALS),
+            ("tags: ", s.get("tags", {}), q_tuple[1], BAD_VALS),
+            ("disposition: ", s.get("disposition", {}), q_tuple[2], BAD_VALS | {0}),
         )
         for prefix, sdict, valid_keys, forbidden in ituple:
             val_list += [
@@ -167,46 +168,9 @@ def print_info(fopts: encode.FileOpts, deep_probe: bool = False) -> None:
     probesfargs = copy.copy(fopts.subtitle)
     probefargs.pop(-2)
     probesfargs.pop(-2)
-    console.print(f"file: {highlight_path(fopts.fpath)}", highlight=False)
-    with concurrent.futures.ThreadPoolExecutor() as executor, console.status(
-        "querying file..."
-    ):
-        vfut = executor.submit(
-            get_stream_list,
-            "v",
-            (InfoKeys.VSTREAMS, InfoKeys.VSTREAM_TAGS, InfoKeys.DISPOSITIONS),
-            probefargs,
-            deep_probe,
-        )
-        afut = executor.submit(
-            get_stream_list,
-            "a",
-            (InfoKeys.ASTREAMS, InfoKeys.ASTREAM_TAGS, InfoKeys.DISPOSITIONS),
-            probefargs,
-            deep_probe,
-        )
-        sfut = executor.submit(
-            get_stream_list,
-            "s",
-            (InfoKeys.SSTREAMS, InfoKeys.SSTREAM_TAGS, InfoKeys.DISPOSITIONS),
-            probesfargs,
-            deep_probe,
-        )
-        fileduration = executor.submit(
-            ffmpeg.probe,
-            "duration",
-            probefargs,
-            probetype=ffmpeg.ProbeType.FORMAT,
-            extraargs="-pretty",
-            deep_probe=deep_probe,
-        )
-    if (duration := fileduration.result()) is not None:
-        console.print("format_duration=[green]" + duration, highlight=False)
-    vlist, alist, slist = vfut.result(), afut.result(), sfut.result()
 
-    def make_columns(
-        slist: list[list[tuple[str, str]]], title: str
-    ) -> rich.columns.Columns:
+    def make_columns(title: str, *args: Any) -> str:
+        slist = get_stream_list(*args)
         table_list = []
         for subindex, vallist in enumerate(slist):
             table = rich.table.Table(
@@ -226,13 +190,54 @@ def print_info(fopts: encode.FileOpts, deep_probe: bool = False) -> None:
                     rich.markup.escape(textwrap.fill(tup[1], 30)),
                 )
             table_list.append(table)
-        return rich.columns.Columns(
-            table_list, align="left", title=rich.text.Text(title, style="bold italic")
-        )
+        with console.capture() as capture:
+            console.print(
+                rich.columns.Columns(
+                    table_list,
+                    title=rich.text.Text(title, style="bold italic"),
+                )
+            )
+        return capture.get()
 
-    console.print(make_columns(vlist, "Video"))
-    console.print(make_columns(alist, "Audio"))
-    console.print(make_columns(slist, "Subtitles"))
+    console.print(f"file: {highlight_path(fopts.fpath)}", highlight=False)
+    with concurrent.futures.ThreadPoolExecutor() as executor:
+        fileduration = executor.submit(
+            ffmpeg.probe,
+            "duration",
+            probefargs,
+            probetype=ffmpeg.ProbeType.FORMAT,
+            extraargs="-pretty",
+            deep_probe=deep_probe,
+        )
+        vid_fut = executor.submit(
+            make_columns,
+            "Video",
+            "v",
+            (InfoKeys.VSTREAMS, InfoKeys.VSTREAM_TAGS, InfoKeys.DISPOSITIONS),
+            probefargs,
+            deep_probe,
+        )
+        aud_fut = executor.submit(
+            make_columns,
+            "Audio",
+            "a",
+            (InfoKeys.ASTREAMS, InfoKeys.ASTREAM_TAGS, InfoKeys.DISPOSITIONS),
+            probefargs,
+            deep_probe,
+        )
+        sub_fut = executor.submit(
+            make_columns,
+            "Subtitles",
+            "s",
+            (InfoKeys.SSTREAMS, InfoKeys.SSTREAM_TAGS, InfoKeys.DISPOSITIONS),
+            probesfargs,
+            deep_probe,
+        )
+        if (duration := fileduration.result()) is not None:
+            console.print("format_duration=[green]" + duration, highlight=False)
+        console.out(vid_fut.result(), highlight=False)
+        console.out(aud_fut.result(), highlight=False)
+        console.out(sub_fut.result(), highlight=False)
 
 
 def status_wait(
@@ -305,7 +310,6 @@ def setup_pyffserver_stream(fv: encode.EncodeSession) -> None:
     """Communicate with a pyffserver API to set up encode session."""
     payload = {"key": fv.ev.api_key}
     json_payload = {
-        "srt_passphrase": fv.ev.srt_passphrase,
         "vstandard": fv.ev.vstandard,
         "pixfmt": fv.ev.pix_fmt,
         "astandard": fv.ev.astandard,
@@ -313,10 +317,9 @@ def setup_pyffserver_stream(fv: encode.EncodeSession) -> None:
         "framerate": fv.v("v", "r_frame_rate"),
         "keyframe_int": fv.ev.kf_int,
         "keyframe_sec": fv.ev.kf_sec,
-        "fragment_sec": fv.ev.frag_sec,
         "inputbitrate": fv.ev.vbitrate,
         "inputabitrate": fv.ev.abitrate,
-        "copyvideo": str(fv.ev.copy_video).lower(),
+        "use_timeline": str(fv.ev.use_timeline).lower(),
         "bound_w": fv.ev.bound_w,
         "bound_h": fv.ev.bound_h,
     }
@@ -328,7 +331,9 @@ def setup_pyffserver_stream(fv: encode.EncodeSession) -> None:
                     console.print("API key rejected by server")
                     raise SystemExit(1)
                 elif req.status_code != 200:
-                    console.print(f"API server returned status {req.status_code}")
+                    console.print(
+                        f"API server returned error {req.status_code}: {req.reason}"
+                    )
                     console.print("waiting and trying again")
                     time.sleep(1)
                     continue
@@ -356,9 +361,12 @@ def setup_pyffserver_stream(fv: encode.EncodeSession) -> None:
                 "Server returned unexpected error while initiating stream."
             ) from e
         if r.status_code != 200:
-            raise ValueError(
-                f"Server returned error {r.status_code} while initiating stream."
-            )
+            raise ValueError(f"Server returned error {r.status_code}: {r.reason}.")
+        json = r.json()
+        fv.ev.srt_passphrase = json.get("srt_passphrase")
+        fv.ev.endpoint = json.get("endpoint")
+        if not fv.ev.srt_passphrase or not fv.ev.endpoint:
+            raise ValueError("server did not return needed parameters")
 
 
 def start_stream(fv: encode.EncodeSession) -> None:
@@ -366,6 +374,7 @@ def start_stream(fv: encode.EncodeSession) -> None:
     with subprocess.Popen(
         fv.ev.ff_flags,
         text=True,
+        encoding="utf-8",
         stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT,
         env=ffmpeg.ff_bin.env,
@@ -461,13 +470,11 @@ def stream_file(fopts: encode.FileOpts, args: argparse.Namespace) -> None:
         encode.do_framerate_calcs(fv)
         encode.determine_bounds(fv)
         fv.ev.encode_flags = [*encode.get_vflags(fv), *encode.get_aflags(fv)]
-        encode.set_output_flags(fv)
     status_wait(fv, futures)
     encode.close_futures(futures)
     fv.executor.shutdown()
     encode.set_input_flags(fv)
     encode.set_filter_flags(fv)
-    encode.set_ffmpeg_flags(fv)
     if fv.ev.wait:
         with rich.live.Live(
             rich.text.Text("Press Enter to continue...", end=""),
@@ -480,6 +487,8 @@ def stream_file(fopts: encode.FileOpts, args: argparse.Namespace) -> None:
     if fv.ev.pyffserver and not fv.ev.outfile:
         with console.status("connecting to server"):
             setup_pyffserver_stream(fv)
+    encode.set_output_flags(fv)
+    encode.set_ffmpeg_flags(fv)
     start_stream(fv)
 
 
@@ -525,21 +534,11 @@ def parse_files(args: argparse.Namespace, parser: argparse.ArgumentParser) -> No
                 parser.error("input files must be files or directories")
         if not stream_flist:
             parser.error("must supply a directory with at least one file")
-        if (
-            args.files[-1].is_file()
-            and (
-                idx := next(
-                    (
-                        i
-                        for i, val in enumerate(stream_flist[:-1])
-                        if val.samefile(stream_flist[-1])
-                    ),
-                    None,
-                )
+        if args.files[-1].is_file():
+            not_same = lambda f: not f.samefile(stream_flist[-1])  # noqa: E731
+            stream_flist = (
+                list(itertools.dropwhile(not_same, stream_flist))[:-1] or stream_flist
             )
-            is not None
-        ):
-            stream_flist = stream_flist[idx:-1]
         if args.playlist:
             with console.status("constructing playlist..."):
                 add_duration = bool(
@@ -551,11 +550,7 @@ def parse_files(args: argparse.Namespace, parser: argparse.ArgumentParser) -> No
                 playlist = ffmpeg.make_playlist(
                     stream_flist, args.tempdir, add_duration, args.deep_probe
                 )
-            process_file(
-                playlist,
-                args,
-                stream_flist,
-            )
+            process_file(playlist, args, stream_flist)
         else:
             for i, path in enumerate(stream_flist):
                 process_file(path, args, stream_flist)
@@ -637,7 +632,9 @@ def download_win_ffmpeg(dltype: str = "git") -> bool:
         try:
             r = s.get("https://api.github.com/repos/BtbN/FFmpeg-Builds/releases/latest")
             if r.status_code != 200:
-                logger.error(f"Fetch failed, server returned response {r.status_code}")
+                logger.error(
+                    f"Fetch failed, server returned error {r.status_code}: {r.reason}"
+                )
                 return False
         except requests.exceptions.RequestException:
             logger.error("Fetch failed, requests could not communicate with server.")
@@ -657,7 +654,7 @@ def download_win_ffmpeg(dltype: str = "git") -> bool:
                 dir_name = match.group("dir")
                 ff_url = ass["browser_download_url"]
                 break
-        if not ff_url:
+        else:
             logger.error("Could not fetch ffmpeg download URL from github")
             return False
         try:
@@ -690,7 +687,8 @@ def download_win_ffmpeg(dltype: str = "git") -> bool:
                     progress.update(task_id, advance=len(chunk))
                 if response.status_code != 200:
                     logger.error(
-                        f"Fetch failed, server returned response {response.status_code}"
+                        f"Fetch failed, server returned error {response.status_code}:"
+                        f" {response.reason}"
                     )
                     return False
         except requests.exceptions.RequestException:
@@ -786,7 +784,7 @@ def get_parserconfig(
         "path": lambda val: pathlib.Path(val),  # pylint: disable=W0108
     }
     parsed_config = configparser.ConfigParser(converters=converters)
-    read_configs = parsed_config.read(conf_list)
+    read_configs = parsed_config.read(conf_list, encoding="utf-8")
 
     if "pyffstream" in parsed_config.sections():
         main_conf = parsed_config["pyffstream"]
@@ -816,7 +814,7 @@ def get_parserconfig(
         env_config = configparser.ConfigParser()
         # https://github.com/python/mypy/issues/2427
         env_config.optionxform = str  # type: ignore
-        env_config.read(conf_list)
+        env_config.read(conf_list, encoding="utf-8")
         for key in env_config["env"]:
             config.env[key] = env_config["env"][key]
 
@@ -1330,21 +1328,25 @@ def main() -> None:
     elif args.x264:
         args.vencoder = "libx264"
 
-    if args.vencoder not in (
-        encode.StaticEncodeVars.VIDEO_ENCODERS & ffmpeg.ff_bin.vencoders
-    ):
-        parser.error(
-            f"selected vencoder {args.vencoder!r} not supported by ffmpeg installation"
-        )
+    if not args.print_info:
+        if args.vencoder not in (
+            encode.StaticEncodeVars.VIDEO_ENCODERS & ffmpeg.ff_bin.vencoders
+        ):
+            parser.error(
+                f"selected vencoder {args.vencoder!r} not supported by ffmpeg"
+                " installation"
+            )
 
-    if args.soxr and "--enable-libsoxr" not in ffmpeg.ff_bin.build_config:
-        parser.error("soxr specified, but using an ffmpeg build without support")
+        if args.soxr and "--enable-libsoxr" not in ffmpeg.ff_bin.build_config:
+            parser.error("soxr specified, but using an ffmpeg build without support")
 
-    if args.zscale and "zscale" not in ffmpeg.ff_bin.filters:
-        parser.error("zscale specified, but using an ffmpeg build without support")
+        if args.zscale and "zscale" not in ffmpeg.ff_bin.filters:
+            parser.error("zscale specified, but using an ffmpeg build without support")
 
-    if args.fdk and "libfdk_aac" not in ffmpeg.ff_bin.aencoders:
-        parser.error("fdk encoder specified, but using an ffmpeg build without support")
+        if args.fdk and "libfdk_aac" not in ffmpeg.ff_bin.aencoders:
+            parser.error(
+                "fdk encoder specified, but using an ffmpeg build without support"
+            )
 
     if args.obs:
         args.live = True
@@ -1441,11 +1443,11 @@ def main() -> None:
         config_dir.mkdir(parents=True, exist_ok=True)
         config_path = config_dir / f"{APPNAME}.conf"
         new_config = configparser.ConfigParser()
-        new_config.read(config_path)
+        new_config.read(config_path, encoding="utf-8")
         # keep case for env section
         new_full_config = configparser.ConfigParser()
         new_full_config.optionxform = str  # type: ignore
-        new_full_config.read(config_path)
+        new_full_config.read(config_path, encoding="utf-8")
 
         if "pyffstream" not in new_config.sections():
             new_config.add_section("pyffstream")
@@ -1483,7 +1485,7 @@ def main() -> None:
 
         console.print("Writing defaults to config path:")
         console.print(config_path)
-        with config_path.open(mode="w") as f:
+        with config_path.open(mode="w", encoding="utf-8") as f:
             new_full_config.write(f)
         raise SystemExit(0)
 
