@@ -11,11 +11,17 @@ import logging
 import math
 import pathlib
 import re
-import secrets
-import string
+import statistics
 import subprocess
 import threading
-from collections.abc import Iterable, Mapping, MutableMapping, MutableSequence, Sequence
+from collections.abc import (
+    Iterable,
+    Iterator,
+    Mapping,
+    MutableMapping,
+    MutableSequence,
+    Sequence,
+)
 from typing import Any, ClassVar, Final, NamedTuple, cast
 
 from . import ffmpeg
@@ -385,12 +391,8 @@ class FilterList:
     def get(self, k: str, default: ffmpeg.Filter | str | None = None) -> str | None:
         """An implementation of the get method as in dicts."""
         with self.__lock:
-            if k in self.filts:
-                return str(self.filts[k])
-            elif default is not None:
-                return str(default)
-            else:
-                return None
+            out = self.filts.get(k, default)
+            return None if out is None else str(out)
 
     def __setitem__(self, key: str, val: str | ffmpeg.Filter | Sequence[str]) -> None:
         """Set filter.
@@ -495,7 +497,6 @@ class StaticEncodeVars:
     )
     kf_int: str = "0"
     kf_sec: str = "0"
-    frag_sec: str = "0"
     afilters: str = ""
     filter_complex: str = ""
     input_flags: MutableSequence[str] = dataclasses.field(default_factory=list)
@@ -509,6 +510,7 @@ class StaticEncodeVars:
     ff_deepprobe_flags: Sequence[str] = dataclasses.field(default_factory=list)
     copy_audio: bool = False
     copy_video: bool = False
+    use_timeline: bool = False
     subs: bool = False
     deep_probe: bool = False
     vindex: int = 0
@@ -592,6 +594,7 @@ class StaticEncodeVars:
         evars.deep_probe = args.deep_probe
         evars.copy_audio = args.copy_audio
         evars.copy_video = args.copy_video
+        evars.use_timeline = args.copy_video
         evars.endpoint = args.endpoint
         evars.api_url = args.api_url
         evars.api_key = args.api_key
@@ -628,17 +631,58 @@ class StaticEncodeVars:
         return evars
 
 
+def divide_off(num: int, divisor: int) -> int:
+    while not math.remainder(num, divisor):
+        num //= divisor
+    return num
+
+
 def do_framerate_calcs(fv: EncodeSession) -> None:
     if fv.ev.obs:
         fv.sdv("v", "r_frame_rate", fv.ev.decimate_target)
-    kf_int = round(fv.ev.kf_target_sec) * round(
-        fractions.Fraction(fv.v("v", "r_frame_rate"))
-    )
-    kf_sec = kf_int / fractions.Fraction(fv.v("v", "r_frame_rate"))
-    frag_sec = kf_sec / (2 * round(kf_sec))
-    fv.ev.kf_int = str(kf_int)
-    fv.ev.kf_sec = f"{float(kf_sec):.8f}".rstrip("0").rstrip(".")
-    fv.ev.frag_sec = f"{float(frag_sec):.8f}".rstrip("0").rstrip(".")
+    framerate = fractions.Fraction(fv.v("v", "r_frame_rate"))
+    if fv.ev.copy_video:
+        fv.ev.use_timeline = True
+        frame_json = ffmpeg.probe(
+            "packet=pts,flags",
+            fv.streamvals["v"].fileargs,
+            f"v:{fv.ev.vindex}",
+            probetype=ffmpeg.ProbeType.RAW_STREAM,
+            deep_probe=fv.ev.deep_probe,
+        )
+        if frame_json:
+            pts_list = [
+                int(pts)
+                for packet in frame_json.get("packets", {})
+                if str((pts := packet.get("pts", ""))).isdigit()
+                and packet.get("flags", "__")[0] == "K"
+            ]
+            if len(pts_list) >= 2:
+                min_diff = statistics.median_low(
+                    y - x for x, y in zip(pts_list[0:], pts_list[1:])
+                )
+                fv.ev.kf_int = str(int(min_diff / 1000 * framerate))
+                fv.ev.kf_sec = f"{min_diff/1000:.7f}"[:-1].rstrip("0").rstrip(".")
+                logger.debug(f"keyframe interval: {fv.ev.kf_int}")
+                return
+    # see https://trac.ffmpeg.org/ticket/9440
+    ideal_gop = fv.ev.kf_target_sec * framerate
+    num = framerate.numerator
+    num = divide_off(num, 2)
+    num = divide_off(num, 5)
+    gop_size = math.ceil(ideal_gop / num) * num
+    seg_length = gop_size / framerate
+    if (
+        not gop_size
+        or seg_length.denominator > 1_000_000
+        or abs(seg_length - fv.ev.kf_target_sec) > 0.4
+    ):
+        fv.ev.use_timeline = True
+        gop_size = math.ceil(ideal_gop)
+        seg_length = gop_size / framerate
+    fv.ev.kf_int = str(gop_size)
+    fv.ev.kf_sec = f"{float(seg_length):.7f}"[:-1].rstrip("0").rstrip(".")
+    logger.debug(f"keyframe interval: {fv.ev.kf_int}")
 
 
 def determine_autocrop(fv: EncodeSession) -> None:
@@ -676,6 +720,7 @@ def determine_autocrop(fv: EncodeSession) -> None:
     with subprocess.Popen(
         cropargs,
         text=True,
+        encoding="utf-8",
         stdout=subprocess.DEVNULL,
         stderr=subprocess.PIPE,
         env=ffmpeg.ff_bin.env,
@@ -831,6 +876,7 @@ def determine_anormalize(fv: EncodeSession) -> None:
             with subprocess.Popen(
                 normargs,
                 text=True,
+                encoding="utf-8",
                 stdout=subprocess.DEVNULL,
                 stderr=subprocess.PIPE,
                 env=ffmpeg.ff_bin.env,
@@ -857,7 +903,7 @@ def determine_anormalize(fv: EncodeSession) -> None:
                 is not None
             ):
                 if fv.ev.normfile is not None:
-                    fv.ev.normfile.write_text(jsonmatch.group("json"))
+                    fv.ev.normfile.write_text(jsonmatch.group("json"), encoding="utf-8")
                 jsonout = json.loads(jsonmatch.group("json"))
                 fv.norm.setstatus(StatusThread.Code.OTHER, "read json")
                 json_to_normfilt(jsonout)
@@ -867,7 +913,7 @@ def determine_anormalize(fv: EncodeSession) -> None:
                 fv.norm.setstatus(StatusThread.Code.FAILED, "[red]failed")
         elif fv.ev.normfile.is_file():
             fv.norm.setstatus(StatusThread.Code.OTHER, "opening")
-            jsonout = json.loads(fv.ev.normfile.read_text())
+            jsonout = json.loads(fv.ev.normfile.read_text(encoding="utf-8"))
             fv.norm.setstatus(StatusThread.Code.OTHER, "read file")
             json_to_normfilt(jsonout)
         else:
@@ -989,17 +1035,25 @@ def extract_style(
         return None
     header = bytes.fromhex(
         "".join(line[10:49] for line in extradata.splitlines())
-    ).decode("utf-8")
+    ).decode("utf-8", errors="surrogateescape")
     return parse_stylelines(header.splitlines())
 
 
 def extract_styles(
     files: Sequence[pathlib.Path], sindex: int, deep_probe: bool = False
-) -> list[StyleFile]:
-    iargs = [(file, sindex, deep_probe) for file in files]
-    with concurrent.futures.ThreadPoolExecutor() as executor:
-        futures = executor.map(lambda p: extract_style(*p), iargs)
-    return [stylefile for stylefile in futures if stylefile is not None]
+) -> Iterator[StyleFile]:
+    executor = concurrent.futures.ThreadPoolExecutor()
+    futures = [
+        executor.submit(extract_style, file, sindex, deep_probe) for file in files
+    ]
+    executor.shutdown(wait=False)
+
+    def style_iterator() -> Iterator[StyleFile]:
+        for future in concurrent.futures.as_completed(futures):
+            if (result := future.result()) is not None:
+                yield result
+
+    return style_iterator()
 
 
 def get_textsub_list(fv: EncodeSession) -> list[ffmpeg.Filter | str]:
@@ -1035,6 +1089,7 @@ def get_textsub_list(fv: EncodeSession) -> list[ffmpeg.Filter | str]:
         with subprocess.Popen(
             subargs,
             text=True,
+            encoding="utf-8",
             stdout=subprocess.DEVNULL,
             stderr=subprocess.PIPE,
             env=ffmpeg.ff_bin.env,
@@ -1059,11 +1114,8 @@ def get_textsub_list(fv: EncodeSession) -> list[ffmpeg.Filter | str]:
             return []
         if fv.ev.is_playlist and fv.fv("s", "codec_name") == "ass":
             fv.subs.setstatus(StatusThread.Code.OTHER, "merging")
-            stylefuture = fv.executor.submit(
-                extract_styles,
-                fv.fopts.allpaths,
-                cast(int, fv.ev.sindex),
-                fv.ev.deep_probe,
+            stylegen = extract_styles(
+                fv.fopts.allpaths, cast(int, fv.ev.sindex), fv.ev.deep_probe
             )
             subass = fv.ev.tempdir / "subs.ass"
             # fmt: off
@@ -1091,22 +1143,20 @@ def get_textsub_list(fv: EncodeSession) -> list[ffmpeg.Filter | str]:
                     f" {result.returncode}"
                 )
                 return []
-            try:
-                sublines = subass.read_text().splitlines()
-            except UnicodeDecodeError as e:
-                fv.subs.setstatus(StatusThread.Code.FAILED, "[red]failed")
-                logger.warning(f"reading extracted merged subtitles failed:\n{e}")
-                return []
+            sublines = subass.read_text(
+                encoding="utf-8-sig", errors="surrogateescape"
+            ).splitlines()
             mainstyle = parse_stylelines(sublines)
             if mainstyle is not None:
-                styles = stylefuture.result()
-                for style in styles:
+                for style in stylegen:
                     for i, name in enumerate(style.names):
                         if name not in mainstyle.names:
                             sublines.insert(mainstyle.insert_index, style.lines[i])
                             mainstyle.names.append(name)
-                sublines = [line + "\n" for line in sublines]
-                subass.write_text("".join(sublines))
+                with subass.open(
+                    mode="w", encoding="utf-8", errors="surrogateescape"
+                ) as f:
+                    f.writelines(line + "\n" for line in sublines)
             in_flags = []
             attach_flags = []
             for i, file in enumerate(fv.fopts.allpaths):
@@ -1207,9 +1257,9 @@ def determine_scale(fv: EncodeSession) -> None:
             logger.warning("Could not find all specified shaders in list.")
         if custom_shaders:
             shader_concat = fv.ev.tempdir / "customshaders.glsl"
-            with shader_concat.open(mode="w") as f:
+            with shader_concat.open(mode="w", encoding="utf-8") as f:
                 for shader in custom_shaders:
-                    f.write(shader.read_text())
+                    f.write(shader.read_text(encoding="utf-8"))
             libplacebo.append(
                 f"custom_shader_path={ffmpeg.Filter.full_escape(str(shader_concat))}"
             )
@@ -1485,6 +1535,7 @@ def set_input_flags(fv: EncodeSession) -> None:
         return
     input_flags += [
         *(("-thread_queue_size", "4096") if not fv.ev.live else ()),
+        *(("-noaccurate_seek",) if fv.ev.copy_video else ()),
         *(("-ss", fv.ev.timestamp) if fv.ev.timestamp and not fv.ev.slowseek else ()),
         *(("-re",) if not fv.ev.live and fv.ev.outfile is None else ()),
         *fv.fopts.main,
@@ -1532,10 +1583,6 @@ def set_output_flags(fv: EncodeSession) -> None:
 
 
 def set_srt_flags(fv: EncodeSession) -> None:
-    alphabet = string.ascii_letters + string.digits
-    if fv.ev.pyffserver:
-        fv.ev.srt_passphrase = "".join(secrets.choice(alphabet) for i in range(43))
-        # fv.ev.srt_passphrase = secrets.token_urlsafe(32)
     srt_options = [
         *((f"passphrase={fv.ev.srt_passphrase}",) if fv.ev.srt_passphrase else ()),
         "pbkeylen=32",
@@ -1595,6 +1642,6 @@ def set_ffmpeg_flags(fv: EncodeSession) -> None:
         *fv.ev.encode_flags,
         *fv.ev.output_flags,
     ]
-    logger.debug(f"Encode command:\n{ff_flags}")
+    logger.debug(f"Encode command:\n{ff_flags!r}")
     logger.info(f"Encode command:\n{' '.join(ff_flags)}")
     fv.ev.ff_flags = ff_flags
